import 'package:build_test/build_test.dart';
import 'package:merging_builder/merging_builder.dart';
import 'package:test/test.dart';

import 'src/generators/add_names_generator.dart';

/// Tests if the builder generates the expected output file.
///
/// To run this program navigate to the top directory the package
/// [merging_builder] and use the command:
///
/// # pub run build_runner test -- -r expanded
void main() {
  test('Test: merging_builder. \n Generate standalone file: researchers.dart',
      () async {
    final Map<String, String> sourceAssets = {
      '$_pkgName|lib/input/researcher_a.dart': _researcher_aDotDart,
      '$_pkgName|lib/input/researcher_b.dart': _researcher_bDotDart,
    };
    final builder = MergingBuilder(
      generator: AddNamesGenerator(),
      inputFiles: 'lib/input/*.dart',
      outputFile: 'lib/researchers.dart',
      header: AddNamesGenerator.header,
      footer: AddNamesGenerator.footer,
    );
    await testBuilder(
      builder,
      sourceAssets,
      outputs: {
        '$_pkgName|lib/researchers.dart': _researchersDotDart,
      },
      reader: await PackageAssetReader.currentIsolate(),
    );
  });
}

const String _pkgName = 'merging_builder';

const _researchersDotDart = r'''
// GENERATED CODE. DO NOT MODIFY. Generated by AddNamesGenerator.

/// Added names.
final name0 = ['Thomas', 'Mayor'];
final name1 = ['Philip', 'Martens'];

final List<List<String>> names = [
  ['Thomas', 'Mayor'],
  ['Philip', 'Martens'],
];

/// This is the footer.
''';

const _researcher_aDotDart = r'''
import 'package:merging_builder/src/annotations/add_integers.dart';
import 'package:merging_builder/src/annotations/add_names.dart';
import 'package:merging_builder/src/annotations/add_numbers.dart';

/// Const class for testing purposes.
@AddNames()
@AddNumbers()
@AddIntegers()
class ResearcherA {
  const ResearcherA();

  final List<String> names = const ['Thomas', 'Mayor'];

  final List<int> integers = const [47, 91];

  final num number = 19;

  final String title = 'ResearcherA';
}
''';

const _researcher_bDotDart = r'''
import 'package:merging_builder/src/annotations/add_integers.dart';
import 'package:merging_builder/src/annotations/add_names.dart';
import 'package:merging_builder/src/annotations/add_numbers.dart';

/// Const class for testing purposes.
@AddNumbers()
@AddIntegers()
@AddNames()
class ResearcherB {
  const ResearcherB();

  final List<String> names = const ['Philip', 'Martens'];

  final Set<int> integers = const {7, 9};

  final num number = 119;

  final String title = 'ResearcherB';
}
''';
